class SBMutatorFastWarShell extends Mutator config(SBMutatorFastWarshell);

var config bool bTreatWarHeadLauncherSubClasses;
var config bool bReplaceAllProjectileClasses;
var config bool bReplaceAllAltProjectileClasses;
var config bool bDebug;

// Show INI value only once in debug mode. Set by SBWarShell/SBGuidedWarShell.
var bool bSBWarShellINIShown;
var bool bSBGuidedWarShellINIShown;

const CCheckConfigWarning = "Check your config in SBMutatorFastWarShell.ini!";



function bool CheckReplacement(Actor Other, out byte bSuperRelevant) {
	if (Other.Class == class'WarHeadLauncher' || (bTreatWarHeadLauncherSubClasses && WarHeadLauncher(Other) != None))
	{
		if (bReplaceAllProjectileClasses || Weapon(Other).ProjectileClass == Class'WarShell')
			Weapon(Other).ProjectileClass = Class'SBWarShell';

		if (bReplaceAllAltProjectileClasses || Weapon(Other).AltProjectileClass == Class'GuidedWarShell')
			Weapon(Other).AltProjectileClass = Class'SBGuidedWarShell';
	}
	return true; // keep *Other*
}



static function DebugLog(Actor Caller, string msg) {
	log(Caller @ "DEBUG:" @ msg);
}



static function DebugLogProjectileINIValues(Projectile prj) {
local byte i;

	DebugLog(prj, "== INI values ==");
	DebugLog(prj, "bDebug=" $ prj.GetPropertyText("bDebug"));
	DebugLog(prj, "ProjectileSpeed=" $ prj.GetPropertyText("ProjectileSpeed"));
	if (SBWarShell(prj) != None)
		DebugLog(prj, "ProjectileMaxSpeed=" $ prj.GetPropertyText("ProjectileMaxSpeed"));
	DebugLog(prj, "LaunchSoundName=" $ prj.GetPropertyText("LaunchSoundName"));
	DebugLog(prj, "ProjectileHealth=" $ prj.GetPropertyText("ProjectileHealth"));
	DebugLog(prj, "UnknownDestroyerName=" $ prj.GetPropertyText("UnknownDestroyerName"));
	DebugLog(prj, "DestroyMessage=" $ prj.GetPropertyText("DestroyMessage"));
	DebugLog(prj, "LaunchSoundName=" $ prj.GetPropertyText("LaunchSoundName"));
	DebugLog(prj, "LaunchMessage=" $ prj.GetPropertyText("LaunchMessage"));
	DebugLog(prj, "bDestroyedBroadcastMsg=" $ prj.GetPropertyText("bDestroyedBroadcastMsg"));
	DebugLog(prj, "bDestroyedBroadcastMsgCritical=" $ prj.GetPropertyText("bDestroyedBroadcastMsgCritical"));
	if (SBWarShell(prj) != None)
		for (i = 0; i < ArrayCount(SBWarShell(prj).ValidWeaponInstigators); i++)
			DebugLog(prj, "ValidWeaponInstigators[" $ i $ "]" $ "=" $ SBWarShell(prj).ValidWeaponInstigators[i]);
	else
		for (i = 0; i < ArrayCount(SBGuidedWarShell(prj).ValidWeaponInstigators); i++)
			DebugLog(prj, "ValidWeaponInstigators[" $ i $ "]" $ "=" $ SBGuidedWarShell(prj).ValidWeaponInstigators[i]);
	DebugLog(prj, "== /INI values ==");

}



static function bool FindMyMutator(Projectile P, out SBMutatorFastWarShell MyMutator) {
/******************************************************************************
Returns TRUE if *SBMutatorFastWarShell* is found. *MyMutator* is set
accordingly then.

Used by *SBGuidedWarShell* and *SBWarShell*
******************************************************************************/
local Mutator M;

	if (P.level.Game != None)
		for (M = P.level.Game.basemutator; M != None; M = M.NextMutator)
			if (M.class == class'SBMutatorFastWarShell')
			{
				MyMutator = SBMutatorFastWarShell(M);
				return true;
			}
	return false;
}



static function int GetMaxSpeed(int ProjectileSpeed, int MaxSpeedFromIni, string ProjectileName) {
/******************************************************************************
Returns *MaxSpeedFromIni* if it is not below *ProjectileSpeed*.
Otherwise a warning is issued and *ProjectileSpeed* is returned.

Used by *SBGuidedWarShell* and *SBWarShell*
******************************************************************************/
	if (MaxSpeedFromIni < ProjectileSpeed)
	{
		warn(ProjectileName $ ".ProjectileMaxSpeed" @ MaxSpeedFromIni @ "is lower than its ProjectileSpeed" @ ProjectileSpeed @ "." @ CCheckConfigWarning @ "MaxSpeed is now set to" @ ProjectileSpeed);
		return ProjectileSpeed;
	}
	else
		return MaxSpeedFromIni;
}



static function int GetSpeed(int ProjectileSpeed, int DefaultProjectileSpeed, string ProjectileName) {
/******************************************************************************
Returns *ProjectileSpeed* if it is greater than zero.
Otherwise a warning is issued and *DefaultProjectileSpeed* is returned.

Used by *SBGuidedWarShell* and *SBWarShell*
******************************************************************************/
	if (ProjectileSpeed > 0)
		return ProjectileSpeed;

	warn(ProjectileName $ ".ProjectileSpeed" @ ProjectileSpeed @ "must be a positive number." @ CCheckConfigWarning @ "Speed is now set to" @ DefaultProjectileSpeed);
	return DefaultProjectileSpeed;
}



simulated static Function string GetReplacedText(string template, string k_replacetext, string o_replacetext) {
	ReplaceText(template, "%k", k_replacetext, 1);
	ReplaceText(template, "%o", o_replacetext, 1);
	return template;
}



function PreBeginPlay() {
	Super.PreBeginPlay();
	if (bDebug)
	{
		DebugLog(self, "== INI values in PreBeginPlay() ==");
		DebugLog(self, "bReplaceAllProjectileClasses=" $ bReplaceAllProjectileClasses);
		DebugLog(self, "bReplaceAllAltProjectileClasses=" $ bReplaceAllAltProjectileClasses);
		DebugLog(self, "bTreatWarHeadLauncherSubClasses=" $ bTreatWarHeadLauncherSubClasses);
		DebugLog(self, "== /INI values in PreBeginPlay() ==");
	}
}



static function ProjectileLaunched(Projectile prj, string LaunchSoundName, string LaunchMessage, ENetRole NetRole) {
/******************************************************************************
Used by *SBGuidedWarShell* and *SBWarShell*
******************************************************************************/
local pawn P;
local Sound sndLaunchSound;

	if (prj.Level.NetMode == NM_Client)
		return;

	if (LaunchSoundName != "")
	{
		sndLaunchSound = Sound(DynamicLoadObject(LaunchSoundName, class'Sound'));
		if (sndLaunchSound == None)
			Warn("Sound >" $ LaunchSoundName $ "< could not be loaded." @ CCheckConfigWarning);
	}

	if (LaunchMessage != "")
		prj.BroadcastMessage(GetReplacedText(LaunchMessage, prj.Instigator.GetHumanName(), ""));

	if (sndLaunchSound != None && prj != None && prj.Level != None)
		for (P = prj.Level.PawnList; P != None; P = P.NextPawn) // ForEach(AllActors(Class'PlayerPawn')) does not work in static functions
			if (PlayerPawn(P) != None)
					PlayerPawn(P).ClientPlaySound(sndLaunchSound);
					//PlayerPawn(P).ClientReliablePlaySound(sndLaunchSound); <- no differences
}



static singular function ProjectileTakeDamage(Projectile prj, byte ProjectileHealth, bool bIsValidWeaponInstigator, string InstigatorHumanName, int NDamage, Pawn Destroyer, string UnknownDestroyerName, string DestroyMessage, Vector hitlocation, vector momentum, name damageType, bool bDestroyedBroadcastMsg, bool bDestroyedBroadcastMsgCritical, bool bDebug) {
local pawn pwn;
local string DestroyerName;

	if (bDebug)
		DebugLog(prj, "ProjectileTakeDamage call with NDamage=" $ NDamage $ ", Destroyer=" $ Destroyer $ ", hitlocation=" $ hitlocation $ ", momentum=" $ momentum $ ", damageType=" $ damageType $ ", bIsValidWeaponInstigator=" $ bIsValidWeaponInstigator);

	if (NDamage > ProjectileHealth)
	{
		prj.PlaySound(Sound'Expl03', , 6.0); // same as in WarShell.TakeDamage()
		prj.spawn(class'WarExplosion', , , prj.Location); // same as in WarShell.TakeDamage()
		prj.HurtRadius(prj.Damage, 350.0, prj.MyDamageType, prj.MomentumTransfer, HitLocation); // same as in WarShell.TakeDamage()
		prj.RemoteRole = ROLE_SimulatedProxy; // same as in WarShell.TakeDamage()
		if (bIsValidWeaponInstigator)
			if (bDestroyedBroadcastMsg || bDestroyedBroadcastMsgCritical)
			{
				if (Destroyer != None)
					DestroyerName = Destroyer.GetHumanName();
				else
					DestroyerName = UnknownDestroyerName;
				DestroyMessage = GetReplacedText(DestroyMessage, InstigatorHumanName, DestroyerName);
				if (bDestroyedBroadcastMsg)
					prj.Level.BroadcastMessage(DestroyMessage);
				if (bDestroyedBroadcastMsgCritical) // ForEach(AllActors(Class'PlayerPawn')) does not work in static functions
				{
					pwn = prj.Level.PawnList;
					while (pwn != None )
					{
						if (PlayerPawn(pwn) != None)
							pwn.ClientMessage(DestroyMessage, 'CriticalEvent', True);
						pwn = pwn.nextPawn;
					}
				}
			}
		prj.Destroy();
	}
}



simulated static function int ReplaceText(out string Text, string OutString, string InString, int MaxReplaceCount) {
/******************************************************************************
Does maximal *MaxReplaceCount* replaces of *OutString* with *InString* in the
string *Text*. If you want all occurences replaced, set MaxReplaceCount to a
negative Number.
If *InString* contains *OutString*, it won't get replaced after inserting into
*Text*, because this wuold cause an infinite loop.

Returns the number of replacements.
******************************************************************************/
local int i;
local string RemainingStr, ResultStr;
local int result;

	RemainingStr = Text;
	while (RemainingStr != "")
	{
		i = InStr(RemainingStr, OutString);

		if (i < 0) // no further occurences
			break;
		else
		{
			ResultStr = ResultStr $ left(RemainingStr, i) $ InString;
			RemainingStr = Right(RemainingStr, Len(RemainingStr) - i - Len(OutString));
			result++;
			if (MaxReplaceCount >= 0)
				if (result >= MaxReplaceCount)
					break;
		}
	}

	Text = ResultStr $ RemainingStr;
	return result;
}




defaultproperties {
	bDebug=false
	bReplaceAllProjectileClasses=false
	bReplaceAllAltProjectileClasses=false
	bTreatWarHeadLauncherSubClasses=false
}
